﻿using HeuristicLab.Algorithms.PushGP.Generators;
using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;
using HeuristicLab.Core;
using HeuristicLab.Random;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public abstract class ERCExpression<T> : Expression
    {
        private readonly T value;
        public ERCExpression(string stringRepresentation, T value) : base(stringRepresentation)
        {
            this.value = value;
        }

        protected void Eval(IStack<T> stack)
        {
            stack.Push(this.value);
        }
    }

    public class IntegerERCExpression : ERCExpression<long>
    {
        public IntegerERCExpression() : base("INTEGER.ERC", IntegerGenerator.RandomInteger())
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.IntegerStack);
        }
    }

    public class FloatERCExpression : ERCExpression<double>
    {
        protected bool isInitialized = false;

        public FloatERCExpression() : base("FLOAT.ERC", FloatGenerator.RandomFloat())
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.FloatStack);
        }
    }

    public class BooleanERCExpression : ERCExpression<bool>
    {
        public BooleanERCExpression() : base("BOOLEAN.ERC", BooleanGenerator.RandomBoolean())
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.BooleanStack);
        }
    }

    public class NameERCExpression : ERCExpression<string>
    {
        private static IRandom random = new FastRandom();

        public NameERCExpression() : base("NAME.ERC", NameGenerator.RandomName())
        {
        }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.NameStack);
        }
    }
}
