﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    /// <summary>
    /// Empties the given stack.
    /// </summary>
    /// <typeparam name="T">Stacktype</typeparam>
    public abstract class FlushExpression<T> : Expression
    {
        public FlushExpression(string stringRepresentation) : base(stringRepresentation)
        { }

        public void Eval(IStack<T> stack)
        {
            stack.Clear();
        }
    }

    public class IntegerFlushExpression : FlushExpression<long>
    {
        public IntegerFlushExpression() : base("INTEGER.FLUSH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.IntegerStack);
        }
    }

    public class FloatFlushExpression : FlushExpression<double>
    {
        public FloatFlushExpression() : base("FLOAT.FLUSH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.FloatStack);
        }
    }

    public class BooleanFlushExpression : FlushExpression<bool>
    {
        public BooleanFlushExpression() : base("BOOLEAN.FLUSH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.BooleanStack);
        }
    }

    public class NameFlushExpression : FlushExpression<string>
    {
        public NameFlushExpression() : base("NAME.FLUSH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.NameStack);
        }
    }

    public class ExecFlushExpression : FlushExpression<Expression>
    {
        public ExecFlushExpression() : base("EXEC.FLUSH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.ExecStack);
        }
    }

    public class CodeFlushExpression : FlushExpression<Expression>
    {
        public CodeFlushExpression() : base("CODE.FLUSH")
        { }

        public override void Eval(IInterpreter interpreter)
        {
            this.Eval(interpreter.CodeStack);
        }
    }
}
